import subprocess
import re
import importlib.metadata
from typing import List, Dict, Tuple, Optional

# ANSI 颜色代码
class Colors:
    RED = '\033[91m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    BLUE = '\033[94m'
    MAGENTA = '\033[95m'
    CYAN = '\033[96m'
    RESET = '\033[0m'

def read_requirements(file_path: str) -> List[str]:
    try:
        with open(file_path, 'rb') as f:
            content = f.read()
            try:
                content = content.decode('utf-8')
            except UnicodeDecodeError:
                import chardet
                encoding = chardet.detect(content)['encoding']
                content = content.decode(encoding)
        return [line.strip() for line in content.splitlines() 
                if line.strip() and not line.startswith('#')]
    except FileNotFoundError:
        print(f"{Colors.RED}错误：未找到 {file_path} 文件。{Colors.RESET}")
        return []

def install_module(requirement: str) -> bool:
    print(f"{Colors.CYAN}正在安装: {requirement}{Colors.RESET}")
    try:
        # 显示安装过程
        process = subprocess.run(
            ["pip", "install", requirement],
            check=False,
            text=True
        )
        return process.returncode == 0
    except Exception:
        return False

def get_installed_version(package: str) -> Optional[str]:
    try:
        return importlib.metadata.version(package)
    except importlib.metadata.PackageNotFoundError:
        return None

def parse_requirement(requirement: str) -> Tuple[Optional[str], Optional[str]]:
    match = re.match(
        r'^([a-zA-Z0-9][a-zA-Z0-9\-_.]+)\s*(?:==|>=|<=|!=|<|>|~=)?\s*([^;\s]*)', 
        requirement
    )
    if not match:
        return None, None
    return match.group(1), match.group(2) if match.group(2) else None

def main():
    requirements_file = 'requirements.txt'
    requirements = read_requirements(requirements_file)
    
    # 结果统计
    installed_count = 0
    already_installed = Dict[str, str]  # 包名: 版本
    replaced = Dict[str, Tuple[str, str]]  # 包名: (原版本, 新版本)
    skipped = List[str]  # 原始requirement字符串

    already_installed = {}
    replaced = {}
    skipped = []

    print(f"{Colors.YELLOW}\n=== 开始安装依赖包 ==={Colors.RESET}")

    for requirement in requirements:
        package, version = parse_requirement(requirement)
        if not package:
            skipped.append(requirement)
            continue

        # 检查是否已安装
        installed_version = get_installed_version(package)
        if installed_version:
            if version and installed_version == version:
                already_installed[package] = version
                print(f"{Colors.GREEN}√ 已存在: {package}=={version}{Colors.RESET}")
            continue

        # 尝试安装
        if install_module(requirement):
            installed_count += 1
            new_version = get_installed_version(package)
            if new_version:
                if version and new_version != version:
                    replaced[package] = (version, new_version)
            continue

        # 尝试安装最新版
        if install_module(package):
            installed_count += 1
            new_version = get_installed_version(package)
            if new_version:
                replaced[package] = (version, new_version)
            continue

        skipped.append(requirement)
        print(f"{Colors.RED}× 跳过: {requirement}{Colors.RESET}")

    # 输出总结
    print(f"\n{Colors.YELLOW}=== 安装结果总结 ==={Colors.RESET}")
    print(f"{Colors.GREEN}成功安装: {installed_count} 个包{Colors.RESET}")

    if already_installed:
        print(f"{Colors.BLUE}已存在无需安装: {len(already_installed)} 个包{Colors.RESET}")

    if replaced:
        print(f"\n{Colors.MAGENTA}版本替换情况:{Colors.RESET}")
        for pkg, (old_ver, new_ver) in replaced.items():
            print(f"  {pkg}: {Colors.YELLOW}{old_ver or '未指定'}{Colors.RESET} → {Colors.GREEN}{new_ver}{Colors.RESET}")

    if skipped:
        print(f"\n{Colors.RED}未找到包: {len(skipped)} 个{Colors.RESET}")
        for req in skipped:
            print(f"  {req}")

    print(f"\n{Colors.YELLOW}=== 安装完成 ==={Colors.RESET}")

if __name__ == "__main__":
    main()